from cleo.commands import Command
from maildown import utilities


class InitCommand(Command):
    """
    Configures Maildown for use

    init
        {access-key? : Your AWS Access Key ID}
        {secret-key? : Your AWS Secret key}
        {region? : AWS region to use (defaults to "us-east-1")}
        {aws-config-file? : Path to your AWS config file (defaults to ~/.aws/credentials}
    """
    def handle(self):
        kwargs = dict()
        access_key = self.argument('access-key')
        secret_key = self.argument('secret-key')
        region = self.argument('region')
        aws_config_file = self.argument('aws-config-file')

        if access_key:
            kwargs['access_key'] = access_key

        if secret_key:
            kwargs['secret_key'] = secret_key

        if region:
            kwargs['region'] = region

        if aws_config_file:
            kwargs['aws_config_file'] = aws_config_file

        utilities.login(**kwargs)
        self.info('Successfully set AWS credentials')


class VerifyCommand(Command):
    """
    Verifies your ownership of an email address. Must be done prior to sending any messages

    verify
        {email-address : The email address that you want to verify}
    """
    def handle(self):
        email = self.argument('email-address')
        verified = utilities.verify_address(email)

        if verified:
            self.info('This email address has already been verified')

        else:
            self.info(f'Email sent to {email}. You must click the link in this email to verify ownership before '
                      f'you can send any emails')


class SendCommand(Command):
    """
    Send an email to a list of recipients

    send
        {sender : The source email address (you must have verified ownership)}
        {subject : The subject line of the email}
        {--c|content=? : The content of the email to send}
        {--f|file-path=? : A path to a file containing content to send}
        {--t|theme : A path to a css file to be applied to the email}
        {recipients?* : A list of email addresses to send the mail to}
    """
    def handle(self):
        sender = self.argument('sender')
        subject = self.argument('subject')

        content = self.option('content')
        file_path = self.option('file-path')
        theme = self.option('theme')
        recipients = self.argument('recipients')

        if not recipients:
            self.line('You must supply at least one recipient', 'error')
            return

        if not any([content, file_path]) or all([content, file_path]):
            self.line('You must provide either the content or file_path argument only', 'error')
            return

        kwargs = dict(
            sender=sender,
            subject=subject,
            content=content,
            file_path=file_path,
            to=recipients
        )
        if theme:
            kwargs['theme'] = theme

        utilities.send_message(**kwargs)
        self.info('Messages added to queue')







import os
from typing import Optional
from jinja2 import Template
import mistune
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import html
from premailer import transform


class HighlightRenderer(mistune.Renderer):
    """
    This highlight renderer improves the way code blocks are handled
    """

    def block_code(self, code, lang=None):
        if not lang:
            return "\n<pre><code>%s</code></pre>\n" % mistune.escape(code)
        lexer = get_lexer_by_name(lang, stripall=True)
        formatter = html.HtmlFormatter()
        return highlight(code, lexer, formatter)


def generate_content(
    md_content: str,
    theme: str = os.path.join(os.path.dirname(os.path.abspath(__file__)), "style.css"),
    context: Optional[dict] = None,
):
    """
    Generates the content of an email to be sent. This method actually renders two templates:
    1. The extremely simple local template, which writes the stylesheet, header and user-provided md_content to the
    message
    2.  The result of 1. is also treated as a jinja template, and rendered using the arguments provided in the context
    parameter

    Apart from rendering the template, this method also does two other things:
    1. Applies an additional highlight renderer with better support for code blocks
    2. Uses premailer.transform to bake the css into the HTML
    """
    if not context:
        context = {}

    with open(theme) as f:
        theme = f.read()

    markdown = mistune.Markdown(renderer=HighlightRenderer())

    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "template.jinja2"),) as t:
        template = Template(t.read())
    content = transform(template.render(content=markdown(md_content), stylesheet=theme))

    t = Template(content)
    return t.render(context)



from cleo.application import Application
from maildown import commands

application = Application()

application.add(commands.InitCommand())
application.add(commands.VerifyCommand())
application.add(commands.SendCommand())



import os
import toml
from typing import Optional, Dict, Union, SupportsFloat
import configparser
import boto3
from botocore.exceptions import ClientError
from maildown.renderer import generate_content


def get_client() -> boto3.client:
    config = get_config()

    return boto3.client(
        "ses",
        aws_access_key_id=config.get("access_key"),
        aws_secret_access_key=config.get("secret_key"),
        region_name=config.get("region", "us-east-1"),
    )


def verify_address(email: str) -> bool:
    client = get_client()
    addresses = client.list_verified_email_addresses().get("VerifiedEmailAddresses")

    if email in addresses:
        return True

    client.verify_email_address(EmailAddress=email)
    return False


def verify_auth(
    access_key: str, secret_key: str, region_name: str = "us-east-1"
) -> bool:
    """
    Checks that the given credentials work by executing a simple boto3 command
    """
    client = boto3.client(
        "ses",
        aws_access_key_id=access_key,
        aws_secret_access_key=secret_key,
        region_name=region_name,
    )
    try:
        client.list_configuration_sets()
        return True
    except ClientError:
        return False


def get_config() -> dict:
    """
    Returns the existing configuration from the local
    """
    try:
        with open(os.path.join(os.path.expanduser("~"), "maildown.toml")) as f:
            return toml.loads(f.read())
    except FileNotFoundError:
        pass
    return {}


def write_config(**config: Dict[str, Union[str, SupportsFloat, bool]]) -> None:
    """
    Updates the existing local config with the given additional arguments
    """
    existing = get_config()
    for key, val in config.items():
        existing[key] = val
    with open(os.path.expanduser("~/maildown.toml"), "w") as f:
        f.write(toml.dumps(config))


def login(
    access_key: Optional[str] = None,
    secret_key: Optional[str] = None,
    region_name: str = "us-east-1",
    aws_config_file: str = os.path.expanduser("~/.aws/credentials"),
) -> None:
    """
    Checks your AWS credentials are valid, and stores them locally if so for future re use.

    If you provide the access key/secret key arguments directly to this function, then these credentials will be taken
    in the first instance.

    If these arguments are NOT supplied, then this method will first check to see if the AWS_ACCESS_KEY_ID and
    AWS_SECRET_ACCESS_KEY environmental variables have been set.

    If not, this method will attempt to read the file kept at `aws_config_file`, which is the default location of
    the Amazon CLI config file.

    If this method cannot find credentials via any one of these methods, or if the credentials it does find are invalid,
    then an Exception is raised.

    However, if valid credentials can be found, these are stored locally

    TODO: replace all exceptions raised in this method with Maildown ones
    """

    if not any([access_key, secret_key]):
        access_key = os.environ.get("AWS_ACCESS_KEY_ID")
        secret_key = os.environ.get("AWS_SECRET_ACCESS_KEY")

    if not any([access_key, secret_key]):
        config = configparser.ConfigParser()
        config.read(aws_config_file)
        try:
            access_key = config["default"].get("aws_access_key_id")
            secret_key = config["default"].get("aws_secret_access_key")

        except KeyError:
            raise KeyError(
                f"Cannot find expected keys in config file stored at {aws_config_file}"
            )

    if not all([access_key, secret_key]):
        raise AttributeError(
            "No credentials supplied - you must either provide the `access_key`, and `secret_key` "
            "values, set the environment variables `AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY`, or run "
            "`aws configure` and try again"
        )

    if not verify_auth(access_key, secret_key, region_name):
        raise AttributeError("The supplied credentials are not valid")

    config = get_config()
    config["access_key"] = access_key
    config["secret_key"] = secret_key
    config["region_name"] = region_name
    write_config(**config)


def send_message(
    sender: str,
    subject: str,
    to: list,
    content: Optional[str] = None,
    file_path: Optional[str] = None,
    context: Optional[dict] = None,
    theme=None,
):
    if not context:
        context = {}

    if all([content, file_path]) or not any([content, file_path]):
        raise AttributeError(
            "You must provide either the content or filepath attribute"
        )

    if file_path:
        with open(file_path) as f:
            content = f.read()

    kwargs = dict(md_content=content, context=context)

    if theme:
        kwargs["theme"] = theme

    message = generate_content(**kwargs)

    client = get_client()
    return client.send_email(
        Source=sender,
        Destination=dict(ToAddresses=to),
        Message=dict(
            Body=dict(
                Html=dict(Charset="utf-8", Data=message),
                Text=dict(Charset="utf-8", Data=content),
            ),
            Subject=dict(Charset="utf-8", Data=subject),
        ),
    )



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['maildown']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.9,<2.0',
 'cleo>=0.7.2,<0.8.0',
 'jinja2>=2.10,<3.0',
 'mistune>=0.8.4,<0.9.0',
 'pygments>=2.3,<3.0',
 'requests>=2.21,<3.0',
 'toml>=0.10.0,<0.11.0']

entry_points = \
{'console_scripts': ['maildown = maildown-cli:run']}

setup_kwargs = {
    'name': 'maildown',
    'version': '1.0.0',
    'description': 'A super simple CLI for sending emails',
    'long_description': '# Maildown\n\nA super simple CLI for sending emails\n\n',
    'author': 'Christopher Davies',
    'author_email': 'christopherdavies553@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
